"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const events = require("events");
/**
 * Use this object for representing a service in the application
 * A Service is a singleton in the application, that is init after all others services are created
 *
 * You can use a Service to create Listeners or implement shared behavior between others services
 *
 * @exports
 * @abstract
 * @class Service
 */
class Service extends events.EventEmitter {
    /**
     *
     *
     * @class Service
     * @param {Webda} webda - The main instance of Webda
     * @param {String} name - The name of the service
     * @param {Object} params - The parameters block define in the configuration file
     */
    constructor(webda, name, params) {
        super();
        this._initTime = new Date().getTime();
        this._webda = webda;
        this._name = name;
        this._params = params;
    }
    resolve() {
        this.normalizeParams();
    }
    /**
     * Convert an object to JSON using the Webda json filter
     *
     * @class Service
     * @param {Object} object - The object to export
     * @return {String} The export of the strip object ( removed all attribute with _ )
     */
    toPublicJSON(object) {
        return this._webda.toPublicJSON(object);
    }
    /**
     * Will be called after all the Services are created
     *
     * @param config for the host so you can add your own route here
     * @abstract
     */
    init() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * Handle cleaning of params
     */
    normalizeParams() { }
    reinit(config) {
        return __awaiter(this, void 0, void 0, function* () {
            this._params = config;
            this.normalizeParams();
            this.init();
        });
    }
    /**
     * For future use, while deploying this should be call so the service can create what it needs if necessary
     *
     * @abstract
     */
    install(params) {
        return Promise.resolve();
    }
    /**
     * For future use, while undeploying this should be call so the service can create what it needs if necessary
     *
     * @abstract
     */
    uninstall(params) {
        return Promise.resolve();
    }
    /**
     * Emit the event with data and wait for Promise to finish if listener returned a Promise
     */
    emitSync(event, data) {
        var result;
        var promises = [];
        var listeners = this.listeners(event);
        for (var i in listeners) {
            result = listeners[i](data);
            if (result instanceof Promise) {
                promises.push(result);
            }
        }
        return Promise.all(promises);
    }
    /**
     * Listen to an event as on(...) would do except that it will be asynchronous
     * @param event
     * @param callback
     * @param queue Name of queue to use, can be undefined, queue name are used to define differents priorities
     */
    onAsync(event, callback, queue) {
        this._webda.getService("AsyncEvents").bindAsyncListener(this, event, callback, queue);
    }
    /**
     * Return a webda service
     * @param service name to retrieve
     */
    getService(service) {
        return this._webda.getService(service);
    }
    getTypedService(service) {
        return this.getService(service);
    }
    /**
     * Return the Modda definition if any
     *
     */
    static getModda() { }
    getName() {
        return this._name;
    }
    /**
     * Clean the service data, can only be used in test mode
     *
     * @abstract
     */
    __clean() {
        if (typeof global.it !== "function") {
            throw Error("Only for test purpose");
        }
        return this.___cleanData();
    }
    /**
     * @private
     */
    ___cleanData() {
        return Promise.resolve();
    }
    log(level, ...args) {
        this._webda.log(level, ...args);
    }
}
exports.Service = Service;
//# sourceMappingURL=service.js.map