"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const index_1 = require("../index");
/**
 * This class handle NoSQL storage and mapping (duplication) between NoSQL object
 * TODO Create the mapping documentation
 *
 * It use basic CRUD, and can expose those 4 to through HTTP
 *
 * It emits events :
 *   Store.Save: Before saving the object
 *   Store.Saved: After saving the object
 *   Store.Update: Before updating the object
 *   Store.Updated: After updating the object
 *   Store.Delete: Before deleting the object
 *   Store.Deleted: After deleting the object
 *   Store.Get: When getting the object
 *   Store.Action: When an action will be done on an object
 *   Store.Actioned: When an action has been done on an object
 *
 * Mapping:
 *
 *
 * Parameters
 *
 *  map: { ... }
 *  expose: { // Enable the HTTP exposure
 *      url: '', // The url to expose to by default it is service name in lowercase ( users for example )
 *      restrict: {
 *	       create: true, // Don't expose the POST /users
 *         update: true, // Don't expose the PUT /users/{uuid}
 *         delete: true, // Don't expose the DELETE /users/{uuid}
 *         get: true // Don't expose the GET /users/{uuid}
 *      }
 *   }
 */
class Store extends index_1.Executor {
    constructor() {
        super(...arguments);
        this._reverseMap = [];
        this._cascade = [];
    }
    /** @ignore */
    normalizeParams() {
        this._lastUpdateField = this._params.lastUpdateField || "_lastUpdate";
        this._creationDateField = this._params.creationDateField || "_creationDate";
        this._writeConditionField = this._lastUpdateField;
        let model = this._params.model;
        if (!model) {
            model = "Webda/CoreModel";
        }
        this._model = this._webda.getModel(model);
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.normalizeParams();
            this.initMap(this._params.map);
            if (this._params.index && !(yield this.exists("index"))) {
                let index = { uuid: "index" };
                index[this._lastUpdateField] = new Date();
                yield this.save(index);
            }
        });
    }
    initRoutes() {
        this.normalizeParams();
        if (!this._params.expose) {
            return;
        }
        let expose = this._params.expose;
        if (typeof expose == "boolean") {
            expose = {};
            expose.url = "/" + this._name.toLowerCase();
        }
        else if (typeof expose == "string") {
            expose = {
                url: expose
            };
        }
        else if (typeof expose == "object" && expose.url == undefined) {
            expose.url = "/" + this._name.toLowerCase();
        }
        expose.restrict = expose.restrict || {};
        if (!expose.restrict.create) {
            this._addRoute(expose.url, ["POST"], this.httpCreate, {
                model: this._model.name,
                post: {
                    description: `The way to create a new ${this._model.name} model`,
                    summary: "Create a new " + this._model.name,
                    model: this._model.name,
                    operationId: `create${this._model.name}`,
                    responses: {
                        "200": {
                            model: this._model.name
                        },
                        "400": "Object is invalid",
                        "403": "You don't have permissions",
                        "409": "Object already exists"
                    }
                }
            });
        }
        // Dont even create the route that are restricted so we can ease up the test on handler
        let methods = [];
        if (!expose.restrict.update) {
            methods.push("PUT");
            methods.push("PATCH");
        }
        if (!expose.restrict.get) {
            methods.push("GET");
        }
        if (!expose.restrict.delete) {
            methods.push("DELETE");
        }
        this._exposeUrl = expose.url;
        if (methods.length) {
            this._addRoute(expose.url + "/{uuid}", methods, this.httpRoute, {
                model: this._model.name,
                get: {
                    description: `Retrieve ${this._model.name} model if permissions allow`,
                    summary: "Retrieve a " + this._model.name,
                    operationId: `get${this._model.name}`,
                    responses: {
                        "200": {
                            model: this._model.name
                        },
                        "400": "Object is invalid",
                        "403": "You don't have permissions",
                        "404": "Unknown object"
                    }
                },
                put: {
                    description: `Update a ${this._model.name} if the permissions allow`,
                    summary: "Update a " + this._model.name,
                    model: this._model.name,
                    operationId: `updatet${this._model.name}`,
                    responses: {
                        "200": {
                            model: this._model.name
                        },
                        "400": "Object is invalid",
                        "403": "You don't have permissions",
                        "404": "Unknown object"
                    }
                },
                patch: {
                    description: `Patch a ${this._model.name} if the permissions allow`,
                    summary: "Patch a " + this._model.name,
                    model: this._model.name,
                    operationId: `updatet${this._model.name}`,
                    responses: {
                        "204": "",
                        "400": "Object is invalid",
                        "403": "You don't have permissions",
                        "404": "Unknown object"
                    }
                },
                delete: {
                    operationId: `delete${this._model.name}`,
                    description: `Delete ${this._model.name} if the permissions allow`,
                    summary: "Delete a " + this._model.name,
                    model: this._model.name,
                    responses: {
                        "204": "",
                        "403": "You don't have permissions",
                        "404": "Unknown object"
                    }
                }
            });
        }
        if (this._model && this._model.getActions) {
            let actions = this._model.getActions();
            Object.keys(actions).forEach(name => {
                let action = actions[name];
                action.name = name;
                if (!action.name) {
                    throw Error("Action needs a name got:" + action);
                }
                if (!action.method) {
                    action.method = ["PUT"];
                }
                if (action.global) {
                    // By default will grab the object and then call the action
                    if (!action._method) {
                        if (!this._model["_" + action.name]) {
                            throw Error("Action static method _" + action.name + " does not exist");
                        }
                        action._method = this.httpGlobalAction;
                    }
                    this._addRoute(expose.url + "/" + action.name, action.method, action._method, action.swagger);
                }
                else {
                    // By default will grab the object and then call the action
                    if (!action._method) {
                        if (!this._model.prototype["_" + action.name]) {
                            throw Error("Action method _" + action.name + " does not exist");
                        }
                        action._method = this.httpAction;
                    }
                    this._addRoute(expose.url + "/{uuid}/" + action.name, action.method, action._method, action.swagger);
                }
            });
        }
    }
    getUrl() {
        return this._exposeUrl;
    }
    initModel(object) {
        // Make sure to send a model object
        if (!(object instanceof this._model)) {
            let model = new this._model();
            model.load(object, true);
            object = model;
        }
        object.__store = this;
        for (var i in this._reverseMap) {
            for (var j in object[this._reverseMap[i].property]) {
                object[this._reverseMap[i].property][j] = this._reverseMap[i].store.initModel(object[this._reverseMap[i].property][j]);
            }
        }
        return object;
    }
    addReverseMap(prop, cascade, store) {
        this._reverseMap.push({
            property: prop,
            store: store
        });
        if (cascade) {
            this._cascade.push(cascade);
        }
    }
    isMapped(property) {
        if (property === undefined) {
            return false;
        }
        if (!this._params.map) {
            return false;
        }
        let map = this._params.map;
        for (let prop in map) {
            // No mapped property or not in the object
            if (map[prop].key === undefined) {
                continue;
            }
            if (map[prop].fields.split(",").indexOf(property) >= 0) {
                return true;
            }
        }
        return false;
    }
    _incrementAttribute(uid, prop, value, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    incrementAttribute(uid, prop, value) {
        return __awaiter(this, void 0, void 0, function* () {
            // If value === 0 no need to update anything
            if (value === 0) {
                return Promise.resolve();
            }
            let updateDate = new Date();
            yield this._incrementAttribute(uid, prop, value, updateDate);
            yield this._handleMapFromPartial(uid, updateDate, prop);
            return this.emitSync("Store.PartialUpdate", {
                object_id: uid,
                store: this,
                partial_update: {
                    increment: {
                        value: value,
                        property: prop
                    }
                }
            });
        });
    }
    upsertItemToCollection(uid, prop, item, index = undefined, itemWriteCondition = undefined, itemWriteConditionField = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            if (itemWriteConditionField === undefined) {
                itemWriteConditionField = "uuid";
            }
            let updateDate = new Date();
            yield this._upsertItemToCollection(uid, prop, item, index, itemWriteCondition, itemWriteConditionField, updateDate);
            yield this._handleMapFromPartial(uid, updateDate);
            yield this.emitSync("Store.PartialUpdate", {
                object_id: uid,
                store: this,
                partial_update: {
                    addItem: {
                        value: item,
                        property: prop,
                        index: index
                    }
                }
            });
        });
    }
    _handleMapFromPartial(uid, updateDate, prop = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isMapped(prop) || this.isMapped(this._lastUpdateField)) {
                // Not optimal need to reload the object
                let object = yield this._get(uid);
                let updates = {};
                updates[this._lastUpdateField] = updateDate;
                if (this.isMapped(prop)) {
                    updates[prop] = object.prop;
                }
                yield this.handleMap(object, this._params.map, updates);
            }
        });
    }
    _upsertItemToCollection(uid, prop, item, index, itemWriteCondition, itemWriteConditionField, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    deleteItemFromCollection(uid, prop, index, itemWriteCondition, itemWriteConditionField) {
        return __awaiter(this, void 0, void 0, function* () {
            if (index === undefined || prop === undefined) {
                throw Error("Invalid Argument");
            }
            if (itemWriteConditionField === undefined) {
                itemWriteConditionField = "uuid";
            }
            let updateDate = new Date();
            yield this._deleteItemFromCollection(uid, prop, index, itemWriteCondition, itemWriteConditionField, updateDate);
            yield this._handleMapFromPartial(uid, updateDate);
            yield this.emitSync("Store.PartialUpdate", {
                object_id: uid,
                store: this,
                partial_update: {
                    deleteItem: {
                        value: index,
                        property: prop,
                        index: index
                    }
                }
            });
        });
    }
    _deleteItemFromCollection(uid, prop, index, itemWriteCondition, itemWriteConditionField, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    initMap(map) {
        if (map == undefined || map._init) {
            return;
        }
        for (var prop in map) {
            var reverseStore = (this._webda.getService(prop));
            if (reverseStore === undefined || !(reverseStore instanceof Store)) {
                map[prop]["-onerror"] = "NoStore";
                this.log("WARN", "Can't setup mapping as store \"", prop, "\" doesn't exist");
                continue;
            }
            var cascade = undefined;
            if (map[prop].cascade) {
                cascade = {
                    store: this._name,
                    name: map[prop].target
                };
            }
            reverseStore.addReverseMap(map[prop].target, cascade, this);
        }
    }
    toString() {
        return this._params.type + "[" + this._name + "]";
    }
    generateUid() {
        return uuid.v4();
    }
    /**
     * Save an object
     *
     * @param {Object} Object to save
     * @param {String} Uuid to use, if not specified take the object.uuid or generate one if not found
     * @return {Promise} with saved object
     */
    save(object, uid = object.uuid) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @ignore */
            if (uid === undefined) {
                uid = this.generateUid();
            }
            if (object.uuid === undefined || object.uuid !== uid) {
                object.uuid = uid;
            }
            for (var i in this._reverseMap) {
                if (object[this._reverseMap[i].property] === undefined) {
                    object[this._reverseMap[i].property] = [];
                }
            }
            object[this._creationDateField] = object[this._lastUpdateField] = new Date();
            object = this.initModel(object);
            if (!object.uuid) {
                object.uuid = object.generateUid();
                object[this._creationDateField] = new Date();
                if (!object.uuid) {
                    object.uuid = this.generateUid();
                }
            }
            yield this.emitSync("Store.Save", {
                object: object,
                store: this
            });
            // Handle object auto listener
            yield object._onSave();
            let res = yield this._save(object, uid);
            object = this.initModel(res);
            yield this.emitSync("Store.Saved", {
                object: object,
                store: this
            });
            yield object._onSaved();
            if (this._params.map != undefined) {
                yield this.handleMap(object, this._params.map, "created");
            }
            // Handle index
            if (this._params.index && object.uuid !== "index" && object.uuid) {
                yield this.handleIndex(object, "created");
            }
            return object;
        });
    }
    _save(object, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    patch(object, uid = undefined, reverseMap = true) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.update(object, uid, reverseMap, true);
        });
    }
    /**
     * Update an object
     *
     * @param {Object} Object to save
     * @param {String} Uuid to use, if not specified take the object.uuid or generate one if not found
     * @param {Boolean} reverseMap internal use only, for disable map resolution
     * @return {Promise} with saved object
     */
    update(object, uid = undefined, reverseMap = true, partial = false) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @ignore */
            var saved;
            var loaded;
            if (uid == undefined) {
                uid = object.uuid;
            }
            // Dont allow to update collections from map
            if (this._reverseMap != undefined && reverseMap) {
                for (var i in this._reverseMap) {
                    if (object[this._reverseMap[i].property] != undefined) {
                        delete object[this._reverseMap[i].property];
                    }
                }
            }
            if (Object.keys(object).length === 0) {
                return {};
            }
            let writeCondition;
            if (this._params.lastUpdate) {
                writeCondition = this._lastUpdateField;
            }
            object[this._lastUpdateField] = new Date();
            let load = yield this._get(uid);
            loaded = this.initModel(load);
            yield this.handleMap(loaded, this._params.map, object);
            // Handle index
            if (this._params.index && loaded.uuid !== "index" && loaded.uuid) {
                yield this.handleIndex(loaded, object);
            }
            yield this.emitSync("Store.Update", {
                object: loaded,
                store: this,
                update: object
            });
            yield loaded._onUpdate(object);
            let res;
            if (partial) {
                yield this._patch(object, uid, writeCondition);
                res = object;
            }
            else {
                res = yield this._update(object, uid, writeCondition);
            }
            // Return updated
            for (let i in res) {
                loaded[i] = res[i];
            }
            for (let i in object) {
                loaded[i] = object[i];
            }
            saved = this.initModel(loaded);
            yield this.emitSync("Store.Updated", {
                object: saved,
                store: this
            });
            yield saved._onUpdated();
            return saved;
        });
    }
    getMapper(map, uuid) {
        for (var i = 0; i < map.length; i++) {
            if (map[i]["uuid"] == uuid) {
                return i;
            }
        }
        return -1;
    }
    removeMapper(map, uuid) {
        let i = this.getMapper(map, uuid);
        if (i >= 0) {
            map.splice(i, 1);
            return true;
        }
        return false;
    }
    _handleUpdatedMap(object, map, mapped, store, updates) {
        return __awaiter(this, void 0, void 0, function* () {
            var mapper = {
                uuid: object.uuid
            };
            // Update only if the key field has been updated
            var found = false;
            for (var field in updates) {
                if (map.fields) {
                    let fields = map.fields.split(",");
                    for (let i in fields) {
                        let mapperfield = fields[i];
                        // Create the mapper object
                        if (updates[mapperfield] !== undefined) {
                            mapper[mapperfield] = updates[mapperfield];
                            found = true;
                        }
                        else if (object[mapperfield] !== undefined) {
                            mapper[mapperfield] = object[mapperfield];
                        }
                    }
                }
                // TODO Need to verify also if fields are updated
                if (field == map.key) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                // None of the mapped keys has been modified -> return
                return;
            }
            // check if reference object has changed
            if (updates[map.key] != undefined && mapped.uuid != updates[map.key]) {
                // create mapper
                if (map.fields) {
                    let fields = map.fields.split(",");
                    for (var j in fields) {
                        let mapperfield = fields[j];
                        // Create the mapper object
                        if (updates[mapperfield] !== undefined) {
                            mapper[mapperfield] = updates[mapperfield];
                        }
                        else if (object[mapperfield] !== undefined) {
                            mapper[mapperfield] = object[mapperfield];
                        }
                    }
                }
                let i = this.getMapper(mapped[map.target], object.uuid);
                if (i >= 0) {
                    // Remove the data from old object
                    yield store.deleteItemFromCollection(mapped.uuid, map.target, i, object.uuid, "uuid");
                }
                // Add the data to new object
                yield store.upsertItemToCollection(updates[map.key], map.target, mapper);
            }
            else {
                return this._handleUpdatedMapMapper(object, map, mapped, store, updates);
            }
        });
    }
    _handleUpdatedMapTransferOut(object, map, mapped, store, updates) {
        var update = {};
        update[map.target] = mapped[map.target];
        return store.update(update, mapped.uuid, false).then(() => {
            return this._handleUpdatedMapTransferIn(object, map, store, updates);
        });
    }
    _handleUpdatedMapTransferIn(object, map, store, updates) {
        // TODO Should be update
        return store.get(updates[map.key]).then(mapped => {
            if (mapped == undefined) {
                return Promise.resolve();
            }
            // Enforce the collection if needed
            if (mapped[map.target] == undefined) {
                mapped[map.target] = [];
            }
            return this._handleUpdatedMapMapper(object, map, mapped, store, updates);
        });
    }
    _handleUpdatedMapMapper(object, map, mapped, store, updates) {
        // Update the mapper
        var mapper = {};
        mapper.uuid = object.uuid;
        if (map.fields) {
            var fields = map.fields.split(",");
            for (var field in fields) {
                if (updates[fields[field]] != undefined) {
                    mapper[fields[field]] = updates[fields[field]];
                }
                else {
                    mapper[fields[field]] = object[fields[field]];
                }
            }
        }
        // Remove old reference
        let i = this.getMapper(mapped[map.target], object.uuid);
        // If not found just add it to the collection
        if (i < 0) {
            return store.upsertItemToCollection(mapped.uuid, map.target, mapper);
        }
        // Else update with a check on the uuid
        return store.upsertItemToCollection(mapped.uuid, map.target, mapper, i, object.uuid, "uuid");
    }
    _handleDeletedMap(object, map, mapped, store) {
        // Remove from the collection
        if (mapped[map.target] === undefined) {
            return Promise.resolve();
        }
        let i = this.getMapper(mapped[map.target], object.uuid);
        if (i >= 0) {
            return store.deleteItemFromCollection(mapped.uuid, map.target, i, object.uuid, "uuid");
        }
        return Promise.resolve();
    }
    _handleCreatedMap(object, map, mapped, store) {
        // Add to the object
        var mapper = {};
        mapper.uuid = object.uuid;
        // Add info to the mapped
        if (map.fields) {
            var fields = map.fields.split(",");
            for (var field in fields) {
                mapper[fields[field]] = object[fields[field]];
            }
        }
        return store.upsertItemToCollection(mapped.uuid, map.target, mapper);
    }
    _handleMapProperty(store, object, property, updates) {
        return __awaiter(this, void 0, void 0, function* () {
            let mapped = yield store.get(object[property.key] || updates[property.key]);
            if (mapped == undefined) {
                return;
            }
            if (updates === "created") {
                return this._handleCreatedMap(object, property, mapped, store);
            }
            else if (updates == "deleted") {
                return this._handleDeletedMap(object, property, mapped, store);
            }
            else if (typeof updates == "object") {
                return this._handleUpdatedMap(object, property, mapped, store, updates);
            }
            else {
                return Promise.reject(Error("Unknown handleMap type " + updates));
            }
        });
    }
    _removeAttribute(uuid, attribute) {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    handleIndex(object, updates) {
        return __awaiter(this, void 0, void 0, function* () {
            let mapUpdates = {};
            if (typeof updates === "object") {
                let toUpdate = false;
                for (let i in updates) {
                    if (this._params.index.indexOf(i) >= 0) {
                        toUpdate = true;
                    }
                }
                if (!toUpdate) {
                    return;
                }
            }
            else if (updates === "deleted") {
                yield this._removeAttribute("index", object.uuid);
                return;
            }
            else if (updates === "created") {
                updates = object;
            }
            let mapper = {};
            this._params.index.forEach(id => {
                mapper[id] = updates[id];
            });
            mapUpdates[object.uuid] = mapper;
            mapUpdates[this._lastUpdateField] = new Date();
            yield this._patch(mapUpdates, "index");
        });
    }
    handleMap(object, map, updates) {
        return __awaiter(this, void 0, void 0, function* () {
            let promises = [];
            if (object === undefined) {
                return;
            }
            for (let prop in map) {
                // No mapped property or not in the object
                if (map[prop].key === undefined ||
                    (object[map[prop].key] === undefined &&
                        updates[map[prop].key] === undefined)) {
                    continue;
                }
                let store = this.getService(prop);
                // Cant find the store for this collection
                if (store == undefined) {
                    continue;
                }
                promises.push(this._handleMapProperty(store, object, map[prop], updates));
            }
            return Promise.all(promises);
        });
    }
    _update(object, uid, writeCondition) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    _patch(object, uid, writeCondition) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    cascadeDelete(obj, uuid) {
        return __awaiter(this, void 0, void 0, function* () {
            // We dont need uuid but Binary store will need it
            return this.delete(obj.uuid);
        });
    }
    /**
     * Delete an object
     *
     * @param {String} uuid to delete
     * @param {Boolean} delete sync even if asyncDelete is active
     * @return {Promise} the deletion promise
     */
    delete(uid, sync = false) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @ignore */
            let to_delete;
            if (typeof uid === "object") {
                to_delete = uid;
                uid = to_delete.uuid;
                if (!(to_delete instanceof this._model)) {
                    to_delete = this.initModel(yield this._get(uid));
                }
            }
            else {
                to_delete = this.initModel(yield this._get(uid));
            }
            if (to_delete === undefined) {
                throw 404;
            }
            yield this.emitSync("Store.Delete", {
                object: to_delete,
                store: this
            });
            yield to_delete._onDelete();
            if (this._params.map != undefined) {
                yield this.handleMap(to_delete, this._params.map, "deleted");
            }
            // Handle index
            if (this._params.index && to_delete.uuid !== "index" && to_delete.uuid) {
                yield this.handleIndex(to_delete, "deleted");
            }
            if (this._cascade != undefined && to_delete !== undefined) {
                var promises = [];
                // Should deactiate the mapping in that case
                for (let i in this._cascade) {
                    if (typeof this._cascade[i] != "object" ||
                        to_delete[this._cascade[i].name] == undefined)
                        continue;
                    var targetStore = this.getTypedService(this._cascade[i].store);
                    if (targetStore == undefined)
                        continue;
                    for (var item in to_delete[this._cascade[i].name]) {
                        promises.push(targetStore.cascadeDelete(to_delete[this._cascade[i].name][item], to_delete.uuid));
                    }
                }
                yield Promise.all(promises);
            }
            if (this._params.asyncDelete && !sync) {
                yield this._patch({
                    __deleted: true
                }, uid);
            }
            else {
                yield this._delete(uid);
            }
            yield this.emitSync("Store.Deleted", {
                object: to_delete,
                store: this
            });
            yield to_delete._onDeleted();
        });
    }
    /**
     * Check if an object exists
     * @abstract
     * @params {String} uuid of the object
     */
    exists(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    _delete(uid, writeCondition) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    _get(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    /**
     * Get an object
     *
     * @param {Array} uuid to gets if undefined then retrieve the all table
     * @return {Promise} the objects retrieved ( can be [] if not found )
     */
    getAll(list = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    /**
     * Provide a way to store configuration in store
     * @param {string} id
     * @returns {Promise<Map<string, any>>}
     */
    getConfiguration(id) {
        return __awaiter(this, void 0, void 0, function* () {
            let object = yield this._get(id);
            if (!object) {
                return undefined;
            }
            let result = new Map();
            for (let i in object) {
                if (i === "uuid" || i === this._lastUpdateField || i.startsWith("_")) {
                    continue;
                }
                result[i] = object[i];
            }
            return result;
        });
    }
    /**
     * Get an object
     *
     * @param {String} uuid to get
     * @return {Promise} the object retrieved ( can be undefined if not found )
     */
    get(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @ignore */
            if (!uid) {
                return undefined;
            }
            let object = yield this._get(uid);
            if (!object) {
                return undefined;
            }
            object = this.initModel(object);
            yield this.emitSync("Store.Get", {
                object: object,
                store: this
            });
            yield object._onGet();
            return object;
        });
    }
    find(request = undefined, offset = 0, limit = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.emitSync("Store.Find", {
                request: request,
                store: this,
                offset: offset,
                limit: limit
            });
            let result = this._find(request, offset, limit);
            yield this.emitSync("Store.Found", {
                request: request,
                store: this,
                offset: offset,
                limit: limit,
                results: result
            });
            return result;
        });
    }
    _find(request, offset, limit) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error("Virtual abstract class - concrete only for MixIn usage");
        });
    }
    // ADD THE EXECUTOR PART
    httpCreate(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            let body = ctx.getRequestBody();
            var object = new this._model();
            object.load(body);
            object[this._creationDateField] = new Date();
            yield object.canAct(ctx, "create");
            try {
                yield object.validate(ctx);
            }
            catch (err) {
                this.log("DEBUG", "Object is not valid", err);
                throw 400;
            }
            if (yield this.exists(object.uuid)) {
                throw 409;
            }
            yield this.save(object, object.uuid);
            ctx.write(object);
            yield this.emitSync("Store.WebCreate", {
                values: body,
                object: object,
                store: this
            });
        });
    }
    httpAction(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            let action = ctx
                .getHttpContext()
                .getUrl()
                .split("/")
                .pop();
            let body = ctx.getRequestBody();
            let uuid = ctx.parameter("uuid");
            if (!uuid) {
                throw 400;
            }
            let object = yield this.get(uuid);
            if (object === undefined || object.__deleted) {
                throw 404;
            }
            yield object.canAct(ctx, action);
            yield this.emitSync("Store.Action", {
                action: action,
                object: object,
                store: this,
                body,
                params: ctx.getRequestParameters()
            });
            let res = yield object["_" + action](ctx);
            if (res) {
                ctx.write(res);
            }
            yield this.emitSync("Store.Actioned", {
                action: action,
                object: object,
                store: this,
                body,
                params: ctx.getRequestParameters()
            });
        });
    }
    httpGlobalAction(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            let body = ctx.getRequestBody();
            let action = ctx
                .getHttpContext()
                .getUrl()
                .split("/")
                .pop();
            yield this.emitSync("Store.Action", {
                action: action,
                store: this,
                body,
                params: ctx.getParameters()
            });
            let res = yield this._model["_" + action](ctx);
            if (res) {
                ctx.write(res);
            }
            yield this.emitSync("Store.Actioned", {
                action: action,
                store: this,
                body,
                params: ctx.getParameters()
            });
        });
    }
    httpUpdate(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            let body = ctx.getRequestBody();
            let uuid = ctx.parameter("uuid");
            body.uuid = uuid;
            let object = yield this.get(uuid);
            if (!object || object.__deleted)
                throw 404;
            yield object.canAct(ctx, "update");
            try {
                yield object.validate(ctx, body);
            }
            catch (err) {
                this.log("Object invalid", err);
                throw 400;
            }
            if (ctx.getHttpContext().getMethod() === "PATCH") {
                let updateObject = new this._model();
                updateObject.load(body);
                yield this.patch(updateObject, uuid);
                object = undefined;
            }
            else {
                let updateObject = new this._model();
                updateObject.load(body);
                // Copy back the _ attributes
                for (let i in object) {
                    if (i.startsWith("_")) {
                        updateObject[i] = object[i];
                    }
                }
                // Copy back the mappers
                for (var i in this._reverseMap) {
                    updateObject[this._reverseMap[i].property] =
                        object[this._reverseMap[i].property];
                }
                // Add mappers back to
                object = yield this.update(updateObject, uuid);
            }
            ctx.write(object);
            yield this.emitSync("Store.WebUpdate", {
                context: ctx,
                updates: body,
                object: object,
                store: this
            });
        });
    }
    httpGet(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            let uuid = ctx.parameter("uuid");
            if (uuid) {
                let object = yield this.get(uuid);
                if (object === undefined || object.__deleted) {
                    throw 404;
                }
                yield object.canAct(ctx, "get");
                ctx.write(object);
                yield this.emitSync("Store.WebGet", {
                    context: ctx,
                    object: object,
                    store: this
                });
                ctx.write(object);
            }
            else {
                // List probably
            }
        });
    }
    httpRoute(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            let uuid = ctx.parameter("uuid");
            if (ctx.getHttpContext().getMethod() == "GET") {
                return this.httpGet(ctx);
            }
            else if (ctx.getHttpContext().getMethod() == "DELETE") {
                let object = yield this.get(uuid);
                if (!object || object.__deleted)
                    throw 404;
                yield object.canAct(ctx, "delete");
                // http://stackoverflow.com/questions/28684209/huge-delay-on-delete-requests-with-204-response-and-no-content-in-objectve-c#
                // IOS don't handle 204 with Content-Length != 0 it seems
                // Have trouble to handle the Content-Length on API Gateway so returning an empty object for now
                ctx.write({});
                yield this.delete(uuid);
                yield this.emitSync("Store.WebDelete", {
                    context: ctx,
                    object_id: uuid,
                    store: this
                });
            }
            else if (ctx.getHttpContext().getMethod() === "PUT" ||
                ctx.getHttpContext().getMethod() === "PATCH") {
                return this.httpUpdate(ctx);
            }
        });
    }
}
exports.Store = Store;
//# sourceMappingURL=store.js.map